
/**
 * The CloseFriends app code. It's wrapped in a function
 * to provide namespacing.
 */

function closeFriends() {

  var PROFILE_IMAGE_CACHE_TIMEOUT = 60 * 60 * 1000; // in milliseconds
  var friends = ['ElatedDotCom'];
  var currentFriend = friends[0];

  init();


  /**
   * Initializes the app. Called when the document is ready.
   */

  function init() {

    // Improve scrolling responsiveness
    $.mobile.touchOverflowEnabled = true;

    // If the user already has a friends list in local storage,
    // grab it, as well as the current friend they were viewing previously
    if ( localStorage['friends'] && JSON.parse( localStorage['friends'] ).length > 0 ) {
      friends = JSON.parse( localStorage['friends'] );
      if ( localStorage['currentFriend'] ) currentFriend = localStorage['currentFriend'];
    }

    // Store the friends list and current friend in local storage
    localStorage['friends'] = JSON.stringify( friends );
    localStorage['currentFriend'] = currentFriend;

    // Add some click handlers to various buttons
    $('.manageFriendsButton').live( 'click', manageFriends );
    $('#addFriendConfirm').live( 'click', addFriend );
    $('#done').live( 'click', showTweetPages );

    // Clear the "screen name" field in the "Add Friend" dialog when it opens
    $('#addFriendDialog').live( 'pagebeforeshow', function() {
      $('#screenName').val('');
    } );

    // Create and show the tweet pages
    showTweetPages();
  }


  /**
   * Prepares the DOM, adds the tweet pages, and displays the
   * current friend's tweet page.
   */

  function showTweetPages() {

    // Don't do anything if there are no friends
    if ( friends.length == 0 ) return;

    // Remove all old tweet pages
    $('.tweetPage').remove();

    // Add the new tweet pages
    addPages();

    // Switch to the current friend's tweet page
    $.mobile.changePage( '#' + currentFriend, { transition: 'flip', reverse: true } );
  }


  /**
   * Adds all the tweet pages to the DOM, populating them as it goes.
   * Called from showTweetPages().
   */

  function addPages() {

    // Store a reference to the previously added page
    var prevPage = null;

    // Create each page's markup
    for ( var i=0; i<friends.length; i++ ) addPage( friends[i] );

    // Render the pages
    for ( var i=0; i<friends.length; i++ ) $('#'+friends[i]).page();

    // Add a handler that runs whenever the tweet page is shown...
    $('.tweetPage').bind( 'pageshow', function() {

      // Update the dots at the bottom of the screen to
      // highlight the new current page
      
      $('#dots').empty();

      for ( var i=0; i<friends.length; i++ ) {
        if ( friends[i] == this.id ) {
          $('#dots').append( '<span class="highlight">.</span>' );
        } else {
          $('#dots').append( '.' );
        }
      }

      // Store the page's associated screen name in currentFriend
      currentFriend = localStorage['currentFriend'] = $.mobile.activePage.attr('id');
    } );

    // Hide the dots if there's only one page
    if ( friends.length < 2 ) {
      $('#dots').hide();
    } else {
      $('#dots').show();
    }


    /**
     * Adds a single tweet page to the DOM, and populates it with
     * the info from Twitter.
     * Called from its parent function, addPages().
     */

    function addPage( pageId ) {

      // Don't add the page if it's already in the DOM
      if ( document.getElementById( pageId ) ) return;

      // Create the basic markup for the new page
      var pageMarkup = '<div class="tweetPage" data-role="page" data-theme="a" id="' + pageId + '" data-url="' + pageId + '" data-title="CloseFriends">';
      pageMarkup += '<div class="gloss"></div>';
      pageMarkup += '<div class="tweetPageContent">';
      pageMarkup += '<a href="http://twitter.com/' + pageId + '"><img class="profileImage" src="loading.png" alt="Profile image"></a>';
      pageMarkup += '<h1>@' + pageId + '</h1>';
      pageMarkup += '<div class="tweetTime"></div>';
      pageMarkup += '<a href="#" class="manageFriendsButton" data-role="button" data-icon="info" data-iconpos="notext" data-transition="flip">Manage Friends</a>';
      pageMarkup += '</div>';
      pageMarkup += '<ul data-role="pagination">';
      if ( prevPage ) pageMarkup += '<li class="ui-pagination-prev"><a href="#' + prevPage + '">Prev</a></li>';
      pageMarkup += '</ul>';
      pageMarkup += '</div>';

      // Add the page to the DOM
      $('body').append( $(pageMarkup) );

      // If this isn't the first page we've added,
      // add a  "Next" link to the previous page's nav
      if ( prevPage ) {
        var nextLinkMarkup = '<li class="ui-pagination-next"><a href="#' + pageId + '">Next</a></li>';
        $('#' + prevPage + ' ul:jqmData(role="pagination")').append( nextLinkMarkup );
      }

      // Is there a fresh copy of the friend's profile image URL in local storage?
      if ( !localStorage['profileImage_'+pageId] || ( new Date().getTime() - localStorage['profileImage_'+pageId+'_timestamp'] > PROFILE_IMAGE_CACHE_TIMEOUT ) ) {

        // No: fetch the image URL and add it to the page when fetched.
        // Also store the (possibly changed) image URL and fetch time in local storage.
        $.ajax( {
          url: "http://api.twitter.com/1/users/show.json?screen_name=" + pageId,
          success: function(data) { 
            var profile_image_url = data['profile_image_url'].replace( /_normal/, "" );
            $('#'+pageId+' .tweetPageContent .profileImage').attr( 'src', profile_image_url );
            localStorage['profileImage_'+pageId] = profile_image_url;
            localStorage['profileImage_'+pageId+'_timestamp'] = new Date().getTime();
          },
          dataType: "jsonp"
        } );

      } else {
        // Yes: add the cached image URL to the page
        $('#'+pageId+' .tweetPageContent .profileImage').attr( 'src', localStorage['profileImage_'+pageId] );
      }

      // Get the friend's latest tweet
      
      $.ajax( {
        url: "http://search.twitter.com/search.json?q=from%3A" + pageId,
        success: function(data) { 

          // Add the tweet text to the page
          var latestTweet = data["results"].length ? data["results"][0]["text"] : "No recent tweets found.";
          var exp = /(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/ig;
          latestTweet = latestTweet.replace( exp,"<a href='$1'>$1</a>" ); 
          exp = /\@(\w+)/ig;
          latestTweet = latestTweet.replace( exp,"<a href='http://twitter.com/$1'>@$1</a>" ); 
          $('#'+pageId+' .tweetPageContent').append( '<div class="recentTweet"><blockquote>' + latestTweet + '</div></blockquote></div>' );

          // Add the tweet timestamp to the page
          var tweetTime = new Date( data["results"][0]["created_at"] ).getTime();
          var currentTime = new Date().getTime();
          var tweetTimeDiff = "";
          var tweetTimeString = "";

          if ( currentTime - tweetTime < 60 * 1000 ) {
            tweetTimeDiff = Math.floor( ( currentTime - tweetTime ) / 1000 ) ;
            tweetTimeString = tweetTimeDiff + " second" + ( tweetTimeDiff == 1 ? "" : "s" ) + " ago";
          } else if ( currentTime - tweetTime < 60 * 60 * 1000 ) {
            tweetTimeDiff =  Math.floor( ( currentTime - tweetTime ) / ( 60 * 1000 ) );
            tweetTimeString = tweetTimeDiff + " minute" + ( tweetTimeDiff == 1 ? "" : "s" ) + " ago";
          } else if ( currentTime - tweetTime < 60 * 60 * 24 * 1000 ) {
            tweetTimeDiff =  Math.floor( ( currentTime - tweetTime ) / ( 60 * 60 * 1000 ) );
            tweetTimeString = tweetTimeDiff + " hour" + ( tweetTimeDiff == 1 ? "" : "s" ) + " ago";
          } else {
            tweetTimeDiff =   Math.floor( ( currentTime - tweetTime ) / ( 60 * 60 * 24 * 1000 ) );
            tweetTimeString = tweetTimeDiff + " day" + ( tweetTimeDiff == 1 ? "" : "s" ) + " ago";
          }
            
          $('#'+pageId+' .tweetTime').html( tweetTimeString );
        },
        dataType: "jsonp"
      } );

      // All done - this page now becomes the new "previous page"
      prevPage = pageId;
    }

  }


  /**
   * Displays the "Manage Friends" page when the user clicks the
   * "info" button.
   */

  function manageFriends() {

    // Get the "friends" list view and empty it
    var friendsList = $('#friendsList');
    friendsList.empty();

    // Add the friends to the list view
    for ( var i=0; i<friends.length; i++ ) {
      var li = $( '<li><a href="#" class="view">' + friends[i] + '</a><a href="#" class="delete">Delete</a></li>' );
      friendsList.append( li );
      li.find('a[class="view"]').jqmData( 'screenName', friends[i] ).click( viewFriend );
      li.find('.delete').jqmData( 'screenName', friends[i] ).click( deleteFriend );
    }

    // Display the Manage Friends page and update the list view widget
    $.mobile.changePage( '#manageFriends', { transition: "flip" } );
    friendsList.listview('refresh');
    $('#dots').empty();
  }


  /**
   * Adds a new friend to the user's friends list.
   * Called when the user taps the "Add Friend" button
   * in the "Add Friend" dialog.
   */

  function addFriend() {

    // Get the screen name that the user entered
    var screenName = $('#screenName').val().replace(/\W/ig, "");

    // Does the friend already exist in the list?
    if ( friends.indexOf( screenName ) == -1 ) {

      // No: add the friend to the list
      friends.push( screenName );

      // Close the "Add Friend" dialog
      $('#addFriendDialog').dialog('close');

      // Add an item for the friend in the list view and update the widget
      var li = $( '<li><a href="#" class="view">' + screenName + '</a><a href="#" class="delete">Delete</a></li>' );
      var friendsList = $('#friendsList');
      friendsList.append( li );
      li.find('a[class="view"]').jqmData('screenName', screenName ).click( viewFriend );
      li.find('.delete').jqmData( 'screenName', screenName ).click( deleteFriend );
      friendsList.listview('refresh');

      // Make sure the "Done" button isn't greyed out
      $('#done').css('opacity',1);

      // Store the updated friends list in local storage
      localStorage['friends'] = JSON.stringify( friends );
    } else {

      // Yes: warn the user and do nothing
      alert( "You've already added that friend to your list." );
    }
  }


  /**
   * Deletes a friend from the user's friends list.
   * Called when the user taps a "Delete" button
   * in the "friends" list view.
   */

  function deleteFriend() {

    // Get the delete button that was tapped
    var deleteButton = $(this);

    // Retrieve the array index of the friend to delete
    var friendIndex = friends.indexOf( deleteButton.jqmData('screenName') );

    // Remove the friend from the array
    friends.splice( friendIndex, 1 );

    // Remove the item from the list view and update the widget
    deleteButton.parent().remove();
    $('#friendsList').listview('refresh');

    // If there are no more friends in the list, grey out the Done button
    if ( friends.length == 0 ) $('#done').css('opacity',.5);

    // Store the updated friends list in local storage
    localStorage['friends'] = JSON.stringify( friends );

    // Reset the current friend to the first friend in the list
    currentFriend = localStorage['currentFriend'] = friends[0];
  }


  /**
   * Displays the selected friend's tweet page.
   * Called when the user taps a friend item
   * in the "friends" list view.
   */

  function viewFriend() {
    currentFriend = $(this).jqmData('screenName');
    showTweetPages();
  }


  /**
   * If the user rotates their device, and we're running
   * the app inside Mobile Safari, then reload the page to
   * avoid rendering glitches.
   */

  $(window).orientationchange( function( e ) {
    if ( !window.navigator.standalone ) window.location.reload( false );
  } );  

}

// All set! Run the app.
$( closeFriends );

